#include "main.h"

/*************************************************************************
	Create_Sector
	creates a complete sector.
	Also copies Pack and Sys_Header informations into the
	sector buffer, then reads a packet full of data from
	the input stream into the sector buffer.
*************************************************************************/

static unsigned char data_buffer[MAX_SECTOR_SIZE];

int create_sector (
Sector_struc 	 *sector,
Pack_struc	 *pack,
Sys_header_struc *sys_header,
unsigned int 	 packet_size,
unsigned int     partial_packet_size,
FILE		 *inputstream,
unsigned char 	 type,
unsigned char    subtype,
unsigned char 	 buffer_scale,
unsigned int 	 buffer_size,
unsigned char 	 buffers,
Timecode_struc   *PTS,
Timecode_struc   *DTS,
unsigned char 	 timestamps,
unsigned int     which_streams,
unsigned int     sector_pad,
unsigned int     svcd_offset,
unsigned int     write_broken_link,
unsigned int     write_seq_end,
unsigned int     write_seq_hdr,
unsigned int     sh_length,
//unsigned char    *seq_hdr)
unsigned char    *seq_hdr,
unsigned int     nsyncwords,
unsigned int     firstsync) // last 3 fix Andrew Church pts dvdauthor

{
//  int i, j, data_bytes, numFound, syncFound, firstOffset, pad_size;
//  unsigned char *index, *tmpPtr=NULL;

  int i, j, data_bytes, pad_size; // fix Andrew Church pts dvdauthor
  unsigned char *index;           // fix Andrew Church pts dvdauthor

  unsigned int data_size, stuff_size, k;
  unsigned int l;

  index = sector->buf;
  sector->length_of_sector=0;

  /* Should we copy Pack Header information ? */

  if (pack != NULL)
  {
    if (mplex_type > MPEG_VCD)
      i = MPEG2_PACK_HEADER_SIZE;
    else
      i = MPEG1_PACK_HEADER_SIZE;
    memcpy (index, pack->buf, i);
    index += i;
    sector->length_of_sector += i;
  }

  /* Should we copy System Header information ? */

  if (sys_header != NULL)
  {
    i = sizeof(sys_header->buf);

    /* only one stream? 3 to 6 bytes less in sys header */
    if (!(which_streams & STREAMS_VIDEO))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO1))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO2))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO3))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO4))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO5))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO6))
      i -= 3;
    if (!(which_streams & STREAMS_AUDIO7))
      i -= 3;

    memcpy (index, sys_header->buf, i);
    index += i;
    sector->length_of_sector += i;
  }

  /* write constant packet header data */

  *(index++) = (unsigned char)(PACKET_START)>>16;
  *(index++) = (unsigned char)(PACKET_START & 0x00ffff)>>8;
  *(index++) = (unsigned char)(PACKET_START & 0x0000ff);
  *(index++) = type;

  /* if it's a padding packet handle it here */

  if (type == PADDING_STR)
  {
    i = 0;
    if (which_streams == STREAMS_AUDIO)
      i = 5;
    if (mplex_type > MPEG_VCD)
      data_bytes = packet_size + MPEG2_AFTER_PACKET_LENGTH - i;
    else
      data_bytes = packet_size + MPEG1_AFTER_PACKET_LENGTH - i;
    *(index++) = (unsigned char)(data_bytes >> 8);
    *(index++) = (unsigned char)(data_bytes & 0xff);
    if (mplex_type < MPEG_MPEG2)
    {
      *(index++) = 0x0F;
      for (j = 0; j < data_bytes - 1; j++)
        *(index++) = (unsigned char) STUFFING_BYTE;
    }
    else
      for (j = 0; j < data_bytes; j++)
        *(index++)=(unsigned char) STUFFING_BYTE;
  }
  else
  {
    /* if partial_packet_size != packet_data_size then write a padding packet after */
    /* the data packet to keep the sector length constant. This is done as long as  */
    /* there is enough room for a padding packet (at least 7 bytes), if not, put in */
    /* stuff_size stuffing bytes                                                    */

    if (packet_size != partial_packet_size)
    {
      data_size = partial_packet_size;
      pad_size = packet_size - partial_packet_size;
    }
    else
    {
      data_size = packet_size;
      pad_size = 0;
    }

    /* check if we have enough data, if not increase stuffing/padding */
    i = data_size;

    if (write_seq_hdr && sh_length)
    {
      data_bytes = sh_length;
      for (j = 0; j < data_bytes; j++)
        data_buffer[j] = seq_hdr[j];
    }
    else
      data_bytes = 0;

    if (svcd_offset)
    {
      l = fread(&data_buffer[data_bytes], 1, svcd_offset, inputstream);
      if (ferror(inputstream))
      {
	 DisplayError("Error reading from an input file.");
	 return FALSE;
      }
      data_bytes += l;
      if (l != svcd_offset)
        pad_size += i - l;
      else
      {
        j = data_bytes;
        data_buffer[j++] = 0;
        data_buffer[j++] = 0;
        data_buffer[j++] = 1;
        data_buffer[j++] = 0xB2;  /* user_data_start_code */
        data_buffer[j++] = 0x10;  /* tag_name = scan information */
        data_buffer[j++] = 0x0E;  /* U_length */
        data_buffer[j++] = 0x00;
        data_buffer[j++] = 0x80;
        data_buffer[j++] = 0x80;  /* Previous_I_offset */
        data_buffer[j++] = 0x00;
        data_buffer[j++] = 0x80;
        data_buffer[j++] = 0x80;  /* Next_I_offset */
        data_buffer[j++] = 0x00;
        data_buffer[j++] = 0x80;
        data_buffer[j++] = 0x80;  /* Backward_offset */
        data_buffer[j++] = 0x00;
        data_buffer[j++] = 0x80;
        data_buffer[j++] = 0x80;  /* Forward_offset */
        data_bytes += 18;
      }
    }

    l = fread(&data_buffer[data_bytes], 1, i - data_bytes, inputstream);
    if (ferror(inputstream))
    {
      DisplayError("Error reading from an input file.");
	  return FALSE;
	}
	data_bytes += l;
    if (data_bytes != i)
      pad_size += i - data_bytes;

    if (write_seq_end && pad_size > 4)
    {
      data_buffer[data_bytes++] = 0;
      data_buffer[data_bytes++] = 0;
      data_buffer[data_bytes++] = 1;
      data_buffer[data_bytes++] = 0xB7; /* write in a sequence end code */
      pad_size -= 4;
    }

    if (write_broken_link)
    {
      i = 0;
      k = 0xFFFFFFFF;
      while ((k != GROUP_START) && (i < data_bytes))
        k = (k << 8) | data_buffer[i++];
      if (k == GROUP_START)
        data_buffer[i + 3] = data_buffer[i + 3] | 0x20;
    }

    if (type == PRIVATE_STREAM1)
      data_bytes += 4;

    if (pad_size < 7)
    {
      stuff_size = pad_size;
      pad_size = 0;
    }
    else
      stuff_size = 0;

    if (mplex_type > MPEG_VCD)
    {
      i = 0;
      if (type != 0xE0)
        i = 5;
      *(index++) = (unsigned char)((data_bytes + stuff_size + MPEG2_AFTER_PACKET_LENGTH - i) >> 8);
      *(index++) = (unsigned char)((data_bytes + stuff_size + MPEG2_AFTER_PACKET_LENGTH - i) & 0xff);
      if (mplex_type == MPEG_SVCD)
        *(index++) = 0x81; /* set the original bit */
      else
        *(index++) = 0x80;
      if (timestamps == TIMESTAMPS_NO)
        *(index++) = TIMESTAMPS_NO | buffers;
      else
        *(index++) = (timestamps << 6) | buffers;

      if (type == 0xE0)
        *(index++) = 13 + stuff_size; /* PES_header_data_length */
      else
        *(index++) = 8 + stuff_size; /* PES_header_data_length without DTS's */

      /* should we write PTS, PTS & DTS or nothing at all ? */

      switch (timestamps)
      {
        case MPEG2_TIMESTAMPS_PTS:
          buffer_timecode(PTS, MARKER_JUST_PTS, &index);
          copy_timecode(PTS, &sector->TS);
          break;

        case MPEG2_TIMESTAMPS_PTS_DTS:
          buffer_timecode(PTS, MARKER_PTS, &index);
          buffer_timecode(DTS, MPEG2_MARKER_DTS, &index);
          copy_timecode(DTS, &sector->TS);
          break;
      }

      /* should we write buffer info ? */

      if (buffers)
      {
        *(index++) = 0x1E;
        *(index++) = (unsigned char) (0x40 | (buffer_scale << 5) | (buffer_size >> 8));
        *(index++) = (unsigned char) (buffer_size & 0xff);
      }

      i = stuff_size;
      if (timestamps == TIMESTAMPS_NO)
      {
        if (type == 0xE0)
          i += 10;
        else
          i += 5;
      }
      else
        if ((timestamps == MPEG2_TIMESTAMPS_PTS) && (type == 0xE0))
          i += 5;
      if (!buffers)
        i += 3;
      for (j = 0; j < i; j++)
        *(index++)=(unsigned char) STUFFING_BYTE;
    }
    else
    {
      i = 0;
      if (type != 0xE0)
        i = 5;
      *(index++) = (unsigned char)((data_bytes + stuff_size + MPEG1_AFTER_PACKET_LENGTH - i) >> 8);
      *(index++) = (unsigned char)((data_bytes + stuff_size + MPEG1_AFTER_PACKET_LENGTH - i) & 0xff);

      i = stuff_size;
      if (!buffers)
        i += 2;
      if (timestamps == TIMESTAMPS_NO)
      {
        if (type != 0xE0)
          i += 4;
        else
          i += 9;
      }
      else
        if ((timestamps == MPEG1_TIMESTAMPS_PTS) && (type == 0xE0))
          i += 5;

      for (j = 0; j < i; j++)
        *(index++) = STUFFING_BYTE;

      /* soll Buffer Info angegeben werden ? */
      /* should we write buffer info ? */

      if (buffers)
      {
        *(index++) = (unsigned char) (0x40 | (buffer_scale << 5) | (buffer_size >> 8));
        *(index++) = (unsigned char) (buffer_size & 0xff);
      }
      /* should we write PTS, PTS & DTS or the no timestamp marker ? */

      switch (timestamps)
      {
        case TIMESTAMPS_NO:
          *(index++) = MARKER_NO_TIMESTAMPS;
          break;

        case MPEG1_TIMESTAMPS_PTS:
          buffer_timecode(PTS, MARKER_JUST_PTS, &index);
          copy_timecode(PTS, &sector->TS);
          break;

        case MPEG1_TIMESTAMPS_PTS_DTS:
          buffer_timecode(PTS, MARKER_PTS, &index);
          buffer_timecode(DTS, MPEG1_MARKER_DTS, &index);
          copy_timecode(DTS, &sector->TS);
          break;
      }
    }

    if (type == PRIVATE_STREAM1)
    {
      *(index++) = subtype;
//      tmpPtr = index; // remember for AC3 data
//      *(index++) = 0; // num of AC3 syncwords in packet
//      *(index++) = 0; // hi offset of first AC3 syncword
//      *(index++) = 0; // lo offset of first AC3 syncword (starting at 1)

// fix Andrew Church pts dvdauthor
      *(index++) = nsyncwords;        // num of AC3 syncwords in packet
      *(index++) = firstsync >> 8;    // hi offset of first AC3 syncword
                                      //   (starting at 1)
      *(index++) = firstsync & 0xFF;  // lo offset of first AC3 syncword

      data_bytes -= 4;
    }

    memcpy(index, data_buffer, data_bytes);
//    if (type == PRIVATE_STREAM1)
//    {
//      numFound = 0;
//      syncFound = 0;
//      firstOffset = 0;
//      for (j = 0; j < data_bytes; j++)
//      {
//        switch (syncFound)
//        {
//          case 0:
//            if (index[j] == 0x0b)
//              syncFound = 1;
//            break;
//          case 1:
//            if (index[j] == 0x77)
//            {
//              numFound++;
//              if (numFound == 1)
//                firstOffset = j;
//            }
//            syncFound = 0;
//        }
//      }
//      tmpPtr[0] = (unsigned char) numFound;
//      tmpPtr[1] = (unsigned char) (firstOffset >> 8);
//      tmpPtr[2] = (unsigned char) (firstOffset & 0xff);
//    }
    index += data_bytes;

    if (type == PRIVATE_STREAM1)
    {
      packet_size += 4;
      data_bytes += 4;
    }

    if (pad_size) /* write a padding packet */
    {
      *(index++) = (unsigned char)(PACKET_START)>>16;
      *(index++) = (unsigned char)(PACKET_START & 0x00ffff)>>8;
      *(index++) = (unsigned char)(PACKET_START & 0x0000ff);
      *(index++) = PADDING_STR;
      *(index++) = (unsigned char)((pad_size - 6) >> 8);
      *(index++) = (unsigned char)((pad_size - 6) & 0xff);
      if (mplex_type < MPEG_MPEG2)
      {
        *(index++) = 0x0F;
        for (i = 0; i < pad_size - 7; i++)
          *(index++) = (unsigned char) STUFFING_BYTE;
      }
      else
        for (i = 0; i < pad_size - 6; i++)
          *(index++) = (unsigned char) STUFFING_BYTE;
    }
  }

  if (sector_pad)
  {
    for (k = 0; k < sector_pad; k++)
      *(index++) = 0;
  }

  /* write other struct data */
  i = 0;
  if (((type != 0xE0) && (type != PADDING_STR)) || (which_streams == STREAMS_AUDIO))
    i = 5;
  if (mplex_type > MPEG_VCD)
    sector->length_of_sector += packet_size + sector_pad + PACKET_HEADER_SIZE + MPEG2_AFTER_PACKET_LENGTH - i;
  else
    sector->length_of_sector += packet_size + sector_pad + PACKET_HEADER_SIZE + MPEG1_AFTER_PACKET_LENGTH - i;
  sector->length_of_packet_data = data_bytes;
  return TRUE;
}

/*************************************************************************
	Create_Pack
	erstellt in einem Buffer die spezifischen Pack-Informationen.
	Diese werden dann spaeter von der Sector-Routine nochmals
	in dem Sektor kopiert.

	writes specifical pack header information into a buffer
	later this will be copied from the sector routine into
	the sector buffer
*************************************************************************/

void create_pack (
Pack_struc	 *pack,
Timecode_struc   *SCR,
unsigned int 	 mux_rate)
{
    unsigned char *index;

    index = pack->buf;

    *(index++) = (unsigned char)((PACK_START)>>24);
    *(index++) = (unsigned char)((PACK_START & 0x00ff0000)>>16);
    *(index++) = (unsigned char)((PACK_START & 0x0000ff00)>>8);
    *(index++) = (unsigned char)(PACK_START & 0x000000ff);
    if (mplex_type > MPEG_VCD)
    {
      buffer_timecode(SCR, MPEG2_MARKER_SCR, &index);
      *(index++) = (unsigned char)(mux_rate >> 14);
      *(index++) = (unsigned char)(0xff & (mux_rate >> 6));
      *(index++) = (unsigned char)(0x03 | ((mux_rate & 0x3f) << 2));
      *(index++) = (unsigned char)(RESERVED_BYTE << 3);
    }
    else
    {
      buffer_timecode(SCR, MPEG1_MARKER_SCR, &index);
      *(index++) = (unsigned char)(0x80 | (mux_rate >> 15));
      *(index++) = (unsigned char)(0xff & (mux_rate >> 7));
      *(index++) = (unsigned char)(0x01 | ((mux_rate & 0x7f) << 1));
    }
    copy_timecode(SCR, &pack->SCR);
}


/*************************************************************************
	Create_Sys_Header
	erstelle in einem Buffer die spezifischen Sys_Header
	Informationen. Diese werden spaeter von der Sector-Routine
	nochmals zum Sectorbuffer kopiert.

	writes specifical system header information into a buffer
	later this will be copied from the sector routine into
	the sector buffer
*************************************************************************/

void create_sys_header (
Sys_header_struc *sys_header,
unsigned int	 rate_bound,
unsigned char	 audio_bound,
unsigned char	 fixed,
unsigned char	 CSPS,
unsigned char	 audio_lock,
unsigned char	 video_lock,
unsigned char	 video_bound,

// audio
unsigned char 	 stream1,
unsigned char 	 buffer1_scale,
unsigned int 	 buffer1_size,

// audio1
unsigned char 	 stream2,
unsigned char 	 buffer2_scale,
unsigned int 	 buffer2_size,

// audio2
unsigned char 	 stream3,
unsigned char 	 buffer3_scale,
unsigned int 	 buffer3_size,

// audio 3
unsigned char 	 stream4,
unsigned char 	 buffer4_scale,
unsigned int 	 buffer4_size,

// audio4
unsigned char 	 stream5,
unsigned char 	 buffer5_scale,
unsigned int 	 buffer5_size,

// audio5
unsigned char 	 stream6,
unsigned char 	 buffer6_scale,
unsigned int 	 buffer6_size,

// audio6
unsigned char 	 stream7,
unsigned char 	 buffer7_scale,
unsigned int 	 buffer7_size,

// audio7
unsigned char 	 stream8,
unsigned char 	 buffer8_scale,
unsigned int 	 buffer8_size,

// video
unsigned char 	 stream9,
unsigned char 	 buffer9_scale,
unsigned int 	 buffer9_size,

unsigned int     which_streams)
{
  unsigned char *index;
  unsigned int rate_restriction_flag = 0;
  int streams;

  index = sys_header->buf;

  /* see which streams we are using */

  audio_bound = 0;
  if(which_streams & STREAMS_AUDIO) audio_bound++;
  if(which_streams & STREAMS_AUDIO1) audio_bound++;
  if(which_streams & STREAMS_AUDIO2) audio_bound++;
  if(which_streams & STREAMS_AUDIO3) audio_bound++;
  if(which_streams & STREAMS_AUDIO4) audio_bound++;
  if(which_streams & STREAMS_AUDIO5) audio_bound++;
  if(which_streams & STREAMS_AUDIO6) audio_bound++;
  if(which_streams & STREAMS_AUDIO7) audio_bound++;

//  if ((which_streams & STREAMS_AUDIO) && (which_streams & STREAMS_AUDIO1))
//   audio_bound = 2;

//  if (!(which_streams & STREAMS_AUDIO) && !(which_streams & STREAMS_AUDIO1))
//   audio_bound = 0;

  if (!(which_streams & STREAMS_VIDEO))
    video_bound = 0;

  streams = 0;
  if (which_streams & STREAMS_VIDEO)  streams++;
  if (which_streams & STREAMS_AUDIO)  streams++;
  if (which_streams & STREAMS_AUDIO1) streams++;
  if (which_streams & STREAMS_AUDIO2) streams++;
  if (which_streams & STREAMS_AUDIO3) streams++;
  if (which_streams & STREAMS_AUDIO4) streams++;
  if (which_streams & STREAMS_AUDIO5) streams++;
  if (which_streams & STREAMS_AUDIO6) streams++;
  if (which_streams & STREAMS_AUDIO7) streams++;

  // byte 0
  *(index++) = (unsigned char)((SYS_HEADER_START)>>24);
  // byte 1
  *(index++) = (unsigned char)((SYS_HEADER_START & 0x00ff0000)>>16);
  // byte 2
  *(index++) = (unsigned char)((SYS_HEADER_START & 0x0000ff00)>>8);
  // byte 3
  *(index++) = (unsigned char)(SYS_HEADER_START & 0x000000ff);


#ifdef OLD_CODE
  if (which_streams == STREAMS_V_A_A1) // 1 video, 2 audio
  {
    // byte 4
    *(index++) = (unsigned char)(SYS_HEADER_LENGTH >> 8);
    // byte 5
    *(index++) = (unsigned char)(SYS_HEADER_LENGTH & 0xff);
  }
  else 
  {
    if (which_streams == STREAMS_V_A) // 1 video, 1 audio
    {
      // byte 4
      *(index++) = (unsigned char)((SYS_HEADER_LENGTH-3) >> 8);
      // byte 5
      *(index++) = (unsigned char)((SYS_HEADER_LENGTH-3) & 0xff);
    }
    else // 1 video
    {
      // byte 4
      *(index++) = (unsigned char)((SYS_HEADER_LENGTH-6) >> 8);
      // byte 5
      *(index++) = (unsigned char)((SYS_HEADER_LENGTH-6) & 0xff);
    }
  }
#endif

// new code
	// byte 4
	*(index++) = (unsigned char)((SYS_HEADER_LENGTH - 6 + ( (streams - 1) * 3) ) >> 8);
    // byte 5
	*(index++) = (unsigned char)((SYS_HEADER_LENGTH - 6 + ( (streams - 1) * 3) ) & 0xff);     
// end new code

  // byte 6
  *(index++) = (unsigned char)(0x80 | (rate_bound >>15));
  // byte 7
  *(index++) = (unsigned char)(0xff & (rate_bound >> 7));
  // byte 8
  *(index++) = (unsigned char)(0x01 | ((rate_bound & 0x7f)<<1));
  // byte 9
  *(index++) = (unsigned char)((audio_bound << 2)|(fixed << 1)|CSPS);
  // byte 10
  *(index++) = (unsigned char)((audio_lock << 7)|
               (video_lock << 6)|0x20|video_bound);

  if (mplex_type > MPEG_VCD)
    // byte 7
    *(index++) = (unsigned char)((rate_restriction_flag << 7) | (RESERVED_BYTE >> 1));
  else
    // byte 7
    *(index++) = (unsigned char) RESERVED_BYTE;

  if (which_streams & STREAMS_VIDEO)
  {
    // byte 8
    *(index++) = stream9;
    // byte 9
    *(index++) = (unsigned char) (0xc0 |
                 (buffer9_scale << 5) | (buffer9_size >> 8));

    // byte 10
    *(index++) = (unsigned char) (buffer9_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO)
  {
    *(index++) = stream1;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer1_scale << 5) | (buffer1_size >> 8));
    *(index++) = (unsigned char) (buffer1_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO1)
  {
    *(index++) = stream2;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer2_scale << 5) | (buffer2_size >> 8));
    *(index) = (unsigned char) (buffer2_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO2)
  {
    *(index++) = stream3;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer3_scale << 5) | (buffer3_size >> 8));
    *(index) = (unsigned char) (buffer3_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO3)
  {
    *(index++) = stream4;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer4_scale << 5) | (buffer4_size >> 8));
    *(index) = (unsigned char) (buffer4_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO4)
  {
    *(index++) = stream5;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer5_scale << 5) | (buffer5_size >> 8));
    *(index) = (unsigned char) (buffer5_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO5)
  {
    *(index++) = stream6;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer6_scale << 5) | (buffer6_size >> 8));
    *(index) = (unsigned char) (buffer6_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO6)
  {
    *(index++) = stream7;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer7_scale << 5) | (buffer7_size >> 8));
    *(index) = (unsigned char) (buffer7_size & 0xff);
  }

  if (which_streams & STREAMS_AUDIO7)
  {
    *(index++) = stream8;
    *(index++) = (unsigned char) (0xc0 |
                 (buffer8_scale << 5) | (buffer8_size >> 8));
    *(index) = (unsigned char) (buffer8_size & 0xff);
  }

}
